package com.zfire.mall.manager.controller.admin;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.zfire.mall.manager.exception.RemoteServiceException;
import com.zfire.mall.manager.helper.ResponseHelper;
import com.zfire.mall.manager.logic.admin.AdminRoleLogic;
import com.zfire.mall.manager.plus.entity.AdminRole;
import com.zfire.mall.manager.plus.entity.AdminUser;
import com.zfire.mall.manager.plus.service.AdminRoleService;
import com.zfire.mall.manager.exception.RemoteServiceException;
import com.zfire.mall.manager.helper.ResponseHelper;
import com.zfire.mall.manager.logic.admin.AdminRoleLogic;
import com.zfire.mall.manager.plus.entity.AdminRole;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

@Slf4j
@RestController
@Api(value = "系统角色管理", tags ={"系统角色管理"} )
@RequestMapping(value = "/admin/role", produces = "application/json; charset=utf-8")
public class AdminRoleController {

    @Autowired
    AdminRoleLogic adminRoleLogic;

    @GetMapping("/list")
    @ApiOperation(value = "角色列表")
    public ResponseHelper<IPage<AdminRole>> list(
            HttpServletRequest request,
            @ApiParam(value = "页号",required = true) @RequestParam(required = true) Integer pageNum,
            @ApiParam(value = "页大小",required = true) @RequestParam(required = true) Integer pageSize
    ) throws RemoteServiceException {
        IPage<AdminRole> adminRoleIPage = adminRoleLogic.listPage(request,pageNum, pageSize);
        return ResponseHelper.success(adminRoleIPage);
    }

    @GetMapping("/detail")
    @ApiOperation(value = "角色详情")
    public ResponseHelper<AdminRole> detail(
            @ApiParam(value = "角色id",required = true) @RequestParam(required = true) String adminRoleId
    ) throws RemoteServiceException {
        AdminRole detail = adminRoleLogic.detail(adminRoleId);
        return ResponseHelper.success(detail);
    }

    @PostMapping("/add")
    @ApiOperation(value = "新增角色")
    public ResponseHelper add(
            HttpServletRequest request,
            @RequestBody AdminRole adminRole
    ) throws RemoteServiceException {
        adminRoleLogic.add(request,adminRole);
        return ResponseHelper.success();
    }

    @PostMapping("/update")
    @ApiOperation(value = "修改角色")
    public ResponseHelper update(@RequestBody AdminRole adminRole) throws RemoteServiceException {
        adminRoleLogic.update(adminRole);
        return ResponseHelper.success();
    }

    @PostMapping("/delete")
    @ApiOperation(value = "删除角色")
    public ResponseHelper delete(
            @ApiParam(value = "角色id",required = true) @RequestParam String adminRoleId
    ) throws RemoteServiceException {
        adminRoleLogic.delete(adminRoleId);
        return ResponseHelper.success();
    }

}
