package com.zfire.mall.manager.logic.admin;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.zfire.mall.manager.bean.admin.AdminModuleBean;
import com.zfire.mall.manager.bean.admin.AdminModuleTree;
import com.zfire.mall.manager.bean.admin.AdminModuleVO;
import com.zfire.mall.manager.bean.admin.AdminUserCom;
import com.zfire.mall.manager.exception.RemoteServiceException;
import com.zfire.mall.manager.logic.common.CommonLogic;
import com.zfire.mall.manager.plus.entity.AdminModule;
import com.zfire.mall.manager.plus.entity.AdminModuleFavorites;
import com.zfire.mall.manager.plus.service.AdminModuleFavoritesService;
import com.zfire.mall.manager.plus.service.AdminModuleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;


@Service
public class AdminModuleLogic {

    @Autowired
    AdminModuleService adminModuleService;
    @Autowired
    AdminModuleFavoritesService adminModuleFavoritesService;
    @Autowired
    CommonLogic commonLogic;


    /**
     * 新增模块
     * @param adminModule
     */
    public void add(AdminModuleBean adminModule){
        Integer level = 1;
        if(!adminModule.getParentId().equals("0")){
            AdminModule parent = adminModuleService.getById(adminModule.getParentId());
            if(parent == null){
                throw new RemoteServiceException("请选择正确的父级菜单");
            }
            level = parent.getLevel() + 1;
            parent.updateById();
        }
        Integer count = adminModuleService.lambdaQuery()
                .eq(AdminModule::getCode,adminModule.getCode())
                .eq(AdminModule::getParentId,adminModule.getParentId())
                .count();
        if(count > 0){
            throw new RemoteServiceException("code或者url重复");
        }
        adminModule.setLevel(level);
        adminModule.setCreateTime(new Date());
        adminModule.insert();
        //添加子集
        if(CollectionUtils.isNotEmpty(adminModule.getChildList())){
            for (AdminModule module : adminModule.getChildList()) {
                module.setLevel(adminModule.getLevel() + 1);
                module.setParentId(adminModule.getModuleId());
            }
            adminModuleService.saveBatch(adminModule.getChildList());
        }
    }


    public void update(AdminModule adminModule){
        Integer level = 1;
        if(adminModule.getParentId() != null && !adminModule.getParentId().equals("0")){
            AdminModule parent = adminModuleService.getById(adminModule.getParentId());
            if(parent == null){
                throw new RemoteServiceException("请选择正确的父级菜单");
            }
            level = parent.getLevel() + 1;
        }
        Integer count = adminModuleService.lambdaQuery()
                .ne(AdminModule::getModuleId,adminModule.getModuleId())
                .eq(AdminModule::getCode,adminModule.getCode())
                .eq(AdminModule::getParentId,adminModule.getParentId())
                .count();
        if(count > 0){
            throw new RemoteServiceException("code或者url重复");
        }
        if(adminModule.getType() != 3) {
            adminModule.setLevel(level);
        }
        adminModule.updateById();
    }


    public void delete(String id){
        adminModuleService.removeById(id);
    }


    public AdminModuleVO detail(String id){
        AdminModule adminModule = adminModuleService.getById(id);
        AdminModuleVO adminModuleVO = BeanUtil.copyProperties(adminModule, AdminModuleVO.class);
        //查询子集
        if(adminModule.getType() == 2){
            List<AdminModule> list = adminModuleService.lambdaQuery().eq(AdminModule::getParentId, adminModule.getModuleId()).list();
            List<AdminModuleVO> adminModuleVOS = BeanUtil.copyToList(list, AdminModuleVO.class);
            adminModuleVO.setChildList(adminModuleVOS);
        }
        return adminModuleVO;
    }


    /**
     * 设置常用菜单
     */
    @Transactional
    public void setupFavorites(List<String> adminModuleIds){
        AdminUserCom adminUser = commonLogic.getAdminUser();
        if(adminModuleIds.size() > 10 || adminModuleIds.size() == 0){
            throw new RemoteServiceException("请选择1-10个常用菜单");
        }
        //1.remove all
        adminModuleFavoritesService.lambdaUpdate().eq(AdminModuleFavorites::getAdminUserId,adminUser.getAdminUserId()).remove();
        //2.add
        List<AdminModuleFavorites> list = new ArrayList<>();
        for(String adminModuleId : adminModuleIds){
            AdminModuleFavorites adminModuleFavorites = new AdminModuleFavorites();
            adminModuleFavorites.setAdminModuleId(adminModuleId);
            adminModuleFavorites.setAdminUserId(adminUser.getAdminUserId());
            list.add(adminModuleFavorites);
        }
        adminModuleFavoritesService.saveBatch(list);
    }

    /**
     * 常用菜单列表
     */
    public List<AdminModule> listFavorites(){
        AdminUserCom adminUser = commonLogic.getAdminUser();
        List<AdminModuleFavorites> list = adminModuleFavoritesService.lambdaQuery().eq(AdminModuleFavorites::getAdminUserId, adminUser.getAdminUserId()).list();
        if(list.size() == 0){
            return new ArrayList<>();
        }
        List<String> adminModuleIds = list.stream().map(AdminModuleFavorites::getAdminModuleId).collect(Collectors.toList());
        List<AdminModule> adminModules = adminModuleService.lambdaQuery().in(AdminModule::getModuleId, adminModuleIds).list();
        return adminModules;
    }


}
